; ##############################################################################
; ##############################################################################
; ##############################################################################
; ADMUX  	ADC Multiplexer Selection Register									
; REFS1 REFS0 ADLAR MUX4 MUX3 MUX2 MUX1 MUX0 									
; 																				
;REFS1 REFS0 																	
; 0 0 	AREF, internal Vref turned off											
; 0 1 	AVCC with external capacitor at AREF pin								
; 1 0 	Reserved																
; 1 1 	Internal 1.1V voltage reference with external capacitor at AREF pin		
; 																				
; ADLAR: 		ADC Left Adjust Result											
; 																				
; MUX4:0: 	Analog Channel and Gain Selection Bits								
;  00000 	ADC0																
;  00001 	ADC1																
;  00010 	ADC2																
;  00011 	ADC3																
;  00100 	ADC4																
;  00101 	ADC5																
;  00110 	ADC6																
;  00111 	ADC7																
;  11110 	1.1V (VBG)															
;  11111 	 0 V (GND)															
;-------------------------------------------------------------------------------
;																					
; ADCSRA  ADC Control and Status Register A										
; ADEN: ADC Enable																	
; ADSC: ADC Start Conversion														
; ADATE: ADC Auto Trigger Enable													
; ADIF: ADC Interrupt Flag															
; ADIE: ADC Interrupt Enable														
; ADPS2:0: ADC Prescaler Select Bits												
;																					
; ADPS2 ADPS1 	ADPS0 Division Factor												
;  0 	0 		0 		  2															
;  0 	0 		1 		  2															
;  0 	1 		0 		  4															
;  0 	1 		1 		  8															
;  1 	0 		0 		 16															
;  1 	0 		1 		 32															
;  1 	1 		0 		 64															
;  1 	1 		1 		128 (bei 20MHz)												
;																					
; ADCSRB  ADC Control and Status Register B										
; ACME: Analog Comparator Multiplexer Enable										
; ADTS2 ADTS1 ADTS0 	Trigger Source												
; *0 	0 		0 		Free Running mode											
;  0 	0 		1 		Analog Comparator											
;  0 	1 		0 		External Interrupt Request 0								
;  0 	1 		1 		Timer/Counter0 Compare Match								
;  1 	0 		0 		Timer/Counter0 Overflow										
;  1 	0 		1 		Timer/Counter1 Compare Match B								
;  1 	1 		0 		Timer/Counter1 Overflow										
;  1 	1 		1 		Timer/Counter1 Capture Event								
ADC_INITIALISIERUNG:
; REFERENZ  										
;	ldi temp,((0<<REFS1)|(0<<REFS0)) ; ext REF		
;	ldi temp,((1<<REFS1)|(1<<REFS0)) ; 1,1V			
	ldi temp,((0<<REFS1)|(1<<REFS0)) ; AVCC + KAPAZITT	
	STS ADMUX, temp

; "FREE Running" + Vorteiler (zw. 50kHz...200kHz)	
	ldi temp,((1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADIF)|(0<<ADIE)|(0<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)) 
	STS ADCSRA,temp       

	ldi temp,((0<<ACME)|(0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0)) 
	STS ADCSRB,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-VBG																	
ADC_MESSEN_VBG:
	LDS temp,ADMUX
	andi temp,0b11100000	; Bitmuster
	ori  temp,0b00011110	;  11110 	1.1V (VBG)
;	ori  temp,0b00011111	;  11111 	 0 V (GND)
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_VBG_L),temp1
	STS(adr_ADC_VBG_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC0																	
; bei 22C = 464																
ADC_MESSEN_0:
	; TEMPERATUR_WIDERSTAND_ON	
	sbi (WIDERSTAND_DDR) ,(WIDERSTAND_PIN_NR)	; auf Ausgang	
	sbi (WIDERSTAND_PORT),(WIDERSTAND_PIN_NR)	; HIGH			
	call WAIT_10ms
	LDS temp,ADMUX
	andi temp,0b11100000	; Bitmuster
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_0_L),temp1
	STS(adr_ADC_0_H),temp2
	; TEMPERATUR_WIDERSTAND_OFF	
	cbi (WIDERSTAND_DDR) ,(WIDERSTAND_PIN_NR)	; auf EINGANG	
	cbi (WIDERSTAND_PORT),(WIDERSTAND_PIN_NR)	; LOW			
; Temperatur=(1023-ADC-Offset)*100:Steigung										
	ldi temp1,LOW (1023)	; 1023	
	ldi temp2,HIGH(1023)
	LDS temp3,(adr_ADC_0_L) ; ADC	
	LDS temp4,(adr_ADC_0_H)
	sub temp1,temp3
	sbc temp2,temp4

	LDS temp3,(adr_TEMPERATUR_IST_SYSOFFS_L)
	LDS temp4,(adr_TEMPERATUR_IST_SYSOFFS_H)
	sub temp1,temp3
	sbc temp2,temp4

	ldi temp3,LOW (100)	; mal 100	
	ldi temp4,HIGH(100)
	call FUNKTION_MULTIPLIKATION_16_16_32

	LDS temp5,(adr_TEMPERATUR_IST_STEIGUNG_L)
	LDS temp6,(adr_TEMPERATUR_IST_STEIGUNG_H)
	call FUNKTION_DIVISION_32_16
;-------------------------------------------------------------------------------
; Offset																		
	LDS temp,(adr_TEMPERATUR_IST_OFFSET)
	tst temp
	brpl ADC_MESSEN_0_P
	com temp
	inc temp
	sub temp1,temp
	sbc temp2,NULL
	STS(adr_TEMPERATUR_IST_L),temp1
	STS(adr_TEMPERATUR_IST_H),temp2
	ret
;-------------------------------------------------------------------------------
ADC_MESSEN_0_P:
	add temp1,temp 	; positiv		
	adc temp2,NULL
	STS(adr_TEMPERATUR_IST_L),temp1
	STS(adr_TEMPERATUR_IST_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC1																	
ADC_MESSEN_1:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -1			; ADC1
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_1_L),temp1
	STS(adr_ADC_1_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC2																	
ADC_MESSEN_2:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -2			; ADC2
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_2_L),temp1
	STS(adr_ADC_2_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC3																	
ADC_MESSEN_3:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -3			; ADC3
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_3_L),temp1
	STS(adr_ADC_3_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC4																	
ADC_MESSEN_4:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -4			; ADC4
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_4_L),temp1
	STS(adr_ADC_4_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC5																	
ADC_MESSEN_5:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -5			; ADC5
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_5_L),temp1
	STS(adr_ADC_5_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC6																	
ADC_MESSEN_6:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -6			; ADC6
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_6_L),temp1
	STS(adr_ADC_6_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-ADC7 (DRUCK)															
ADC_MESSEN_7:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -7			; ADC7
	STS ADMUX,temp
	rcall ADC_MESSUNG_TEMP1_TEMP2
	STS(adr_ADC_7_L),temp1
	STS(adr_ADC_7_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ADC-WERT 256 mal einlesen und daraus den Mittelwert bilden					
;																				
; OUT: 	TEMP1 (low)  															
;		TEMP2																	
; 																				
ADC_MESSUNG_TEMP1_TEMP2:
	call wait_1ms			; ev. kleine PAUSE 
	clr temp1	; HILFSREGISTER CLEAR							
	clr temp2
	clr temp3
	clr temp9	; SCHLEIFE initialisieren		
; neuen ADC-Wert lesen	 Wert verwerfen, da meist vom vorherigem ADC-Kanal	
MESSEN_DUMMY:
	LDS temp,(ADCSRA)	; auslesen		
	sbrs temp, ADIF		; skip if...	
	rjmp MESSEN_DUMMY
	ori temp,(1<<ADIF)
	STS(ADCSRA),temp	; ADIF is cleared by writing a logical one to the flag
; neuen ADC-Wert lesen	(Schleife - 256 mal)								
MESSEN_SCHLEIFE:                
	LDS temp,(ADCSRA)	; auslesen		
	sbrs temp, ADIF		; skip if...	
	rjmp MESSEN_SCHLEIFE
	ori temp,(1<<ADIF)
	STS(ADCSRA),temp	; ADIF is cleared by writing a logical one to the flag
	LDS  R0, ADCL 	; immer zuerst LOW Byte lesen					
	LDS  R1, ADCH 	; danach das mittlerweile gesperrte High Byte	
; alle 256 ADC-Werte addieren												
	add temp1,R0     ; addieren  						
	adc temp2,R1     ; addieren ber Carry  				
	adc temp3,NULL    	; addieren ber Carry  				
	dec temp9			; Schleifenzhler MINUS 1			
	brne MESSEN_SCHLEIFE; ==> SCHLEIFE						
; MITTELWERT-BERECHNUNG	(128 addieren)										
	ldi temp,128
	add temp1,temp	     ; addieren  						
	adc temp2,NULL	     ; addieren ber Carry  				
	adc temp3,NULL    	 ; addieren ber Carry  				
; 	Ergebnis durch 256 teilen (temp5+6 nach temp1+2 kopieren)				
	mov temp1,temp2
	mov temp2,temp3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

